"use strict";

require("core-js/modules/es.symbol.to-primitive.js");
require("core-js/modules/es.date.to-primitive.js");
require("core-js/modules/es.symbol.js");
require("core-js/modules/es.symbol.description.js");
require("core-js/modules/es.number.constructor.js");
require("core-js/modules/es.object.define-property.js");
require("core-js/modules/es.object.keys.js");
require("core-js/modules/es.object.get-own-property-descriptor.js");
require("core-js/modules/es.array.for-each.js");
require("core-js/modules/web.dom-collections.for-each.js");
require("core-js/modules/es.object.get-own-property-descriptors.js");
require("core-js/modules/es.object.define-properties.js");
require("core-js/modules/es.symbol.iterator.js");
require("core-js/modules/es.array.iterator.js");
require("core-js/modules/es.string.iterator.js");
require("core-js/modules/web.dom-collections.iterator.js");
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
require("core-js/modules/es.string.repeat.js");
require("core-js/modules/es.array.flat-map.js");
require("core-js/modules/es.array.unscopables.flat-map.js");
require("core-js/modules/es.object.to-string.js");
require("core-js/modules/es.array.filter.js");
require("core-js/modules/es.array.concat.js");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/** get diagonal repeat lines: looks something like
 *   /             \
    |.             .|
    |.  A m  Gm7   .|
     \             /
*/
var getRepeatLines = function getRepeatLines(dims, type) {
  //x and y determine the diagonal line's length
  var y = 5;
  var x = 7;
  var drawX = type === "backward" ? -x : x;
  var moveX = type === "backward" ? dims.xEnd : dims.x;
  return ["M" + moveX + " " + dims.y + " l " + drawX + " " + -y, "M" + moveX + " " + dims.yEnd + " l " + drawX + " " + y];
};

/* get the points in the repetition
 *  |:  A m7   |  Gm7  :|
 * */
var getRepetitionCircles = function getRepetitionCircles(bars, barView) {
  var yDistance = 4; // from y center of bar to point (either point above or below)
  var xDistance = 4; // distance from bar to points

  var getY = function getY(y, type) {
    var fDist = type === "up" ? -yDistance : yDistance;
    return y + barView.bar.height / 2 + fDist;
  };
  var circle = {
    r: 1,
    stroke: "black"
  };
  var getCircles = function getCircles(bar, barView) {
    var x = bar.repeat === "forward" ? bar.dimensions.x + xDistance : bar.dimensions.x + barView.bar.width - xDistance;
    var y1 = getY(bar.dimensions.y, "up");
    var y2 = getY(bar.dimensions.y, "down");
    var circle1 = _objectSpread(_objectSpread({}, circle), {
      cx: x,
      cy: y1
    });
    var circle2 = _objectSpread(_objectSpread({}, circle), {
      cx: x,
      cy: y2
    });
    return [circle1, circle2];
  };
  return bars.filter(function (bar) {
    return !!bar.repeat;
  }).flatMap(function (bar) {
    return getCircles(bar, barView);
  });
};
var getBarLines = function getBarLines(bars, barView) {
  if (!barView.bar || barView.bar.width === undefined || barView.bar.height === undefined || barView.bar.repetitionLineSpace === undefined) throw new TypeError("barView arguments needed: bar.width, bar.height and bar.repetitionLineSpace");
  var barLines = bars.flatMap(function (bar) {
    var dims = {
      x: bar.dimensions.x,
      y: bar.dimensions.y,
      xEnd: bar.dimensions.x + barView.bar.width,
      yEnd: bar.dimensions.y + barView.bar.height
    };
    var line = ["M" + dims.xEnd + " " + dims.y + " l 0 " + barView.bar.height];
    if (bar.endSection) line.push("m" + (dims.xEnd - barView.bar.repetitionLineSpace) + " " + dims.y + " l 0 " + barView.bar.height);
    if (bar.repeat) {
      if (bar.repeat === "forward") {
        line.push("m" + dims.x + " " + dims.y + " l 0 " + barView.bar.height);
      }
      line = line.concat(getRepeatLines(dims, bar.repeat));
    }
    return line;
  });
  return {
    paths: barLines,
    circles: getRepetitionCircles(bars, barView)
  };
};
var _default = getBarLines;
exports["default"] = _default;