"use strict";

require("core-js/modules/es.symbol.to-primitive.js");
require("core-js/modules/es.date.to-primitive.js");
require("core-js/modules/es.symbol.js");
require("core-js/modules/es.symbol.description.js");
require("core-js/modules/es.number.constructor.js");
require("core-js/modules/es.object.define-property.js");
require("core-js/modules/es.object.keys.js");
require("core-js/modules/es.array.filter.js");
require("core-js/modules/es.object.get-own-property-descriptor.js");
require("core-js/modules/es.array.for-each.js");
require("core-js/modules/web.dom-collections.for-each.js");
require("core-js/modules/es.object.get-own-property-descriptors.js");
require("core-js/modules/es.object.define-properties.js");
require("core-js/modules/es.symbol.iterator.js");
require("core-js/modules/es.array.iterator.js");
require("core-js/modules/es.string.iterator.js");
require("core-js/modules/web.dom-collections.iterator.js");
function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformChords = exports.getChordTextScale = void 0;
require("core-js/modules/es.array.map.js");
require("core-js/modules/es.string.sup.js");
require("core-js/modules/es.regexp.exec.js");
require("core-js/modules/es.string.replace.js");
require("core-js/modules/es.array.reduce.js");
require("core-js/modules/es.object.to-string.js");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
var transformChords = function transformChords(bar) {
  var chords = bar.chords.map(function (chord) {
    return !chord.pitch ? chord : _objectSpread(_objectSpread({}, chord), {}, {
      pitch: replaceFlat(chord.pitch),
      chordType: getSpecialSymbol(chord.chordType),
      sup: chord.sup ? replaceFlat(chord.sup.substr(1, chord.sup.length - 2)) : null,
      bass: chord.bass ? replaceFlat(chord.bass.substr(1, chord.bass.length)) : null
    });
  });
  return _objectSpread(_objectSpread({}, bar), {}, {
    chords: chords
  });
};
exports.transformChords = transformChords;
var replaceFlat = function replaceFlat(str) {
  return str.replace(/b/g, "\u266D");
};
var getSpecialSymbol = function getSpecialSymbol(chordType) {
  if (chordType === "maj7") return "\u2206"; //  ∆
  if (chordType === "halfdim") return "\xF8"; //  ø

  return chordType.replace("dim", "\u2218"); // ∘
};

var getChordTextScale = function getChordTextScale(bars, barWidth) {
  /*
  F maj7
  Db maj7
  B halfdim
  F# halfdim
  F 7(sus4,9)
  Eb 7(sus4,9)
  Db major-13th
  Ab 7(#11,9,13)
  Eb 7(sus4,9,13)
  C suspended-fourth
  */
  var standardCharWidth = 17;
  var smallestCharWidth = bars.reduce(function (prev, bar, i) {
    var width = barWidth / bar.chords.length;
    var maxChorStringLength = getLengthFromLongestChordString(bar);
    var charWidth = maxChorStringLength ? width / maxChorStringLength : 0;
    return !prev ? charWidth : prev > charWidth ? charWidth : prev;
  }, null);
  if (standardCharWidth < smallestCharWidth) return 1;
  return smallestCharWidth / standardCharWidth;
};
exports.getChordTextScale = getChordTextScale;
var getLengthFromLongestChordString = function getLengthFromLongestChordString(bar) {
  return bar.chords.reduce(function (prev, chord, j) {
    var chordStringLength = ["pitch", "chordType", "sup"].reduce(function (prev, key) {
      return prev + (chord[key] ? chord[key].length : 1);
    }, 0);
    return !prev || chordStringLength > prev ? chordStringLength : prev;
  }, null);
};